/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>

#include <boost/shared_ptr.hpp>

namespace rci {

class ControlAspect;
typedef boost::shared_ptr<ControlAspect> ControlAspectPtr;

/**
 * Domain object indicating one aspect of control (e.g. position control).
 * @ingroup DTO
 * @todo Should have constructor with ControlAspect vector as third parameter
 * @todo Control Mode combination should be commutative.
 */
class ControlAspect {

public:

    /**
     * Constructor, initializing control to desired mode.
     * @param name Name of the control mode
     * @param ID Identifier for this control mdoe
     * @return Control mode object, set to 0 (disabled)
     */
    ControlAspect(const std::string &name, const std::string &ID);


    /**
     * Explicitly declared cop constructor
     *
     * @param orig Original control aspect to copy
     */
    ControlAspect(const ControlAspect &orig);

    /**
     * Destructor
     */
    virtual ~ControlAspect();

    /**
     * Returns unique identifier of the control mode instance.
     * @return Unique identifier (uuid)
     */
    std::string getID() const;

    /**
     * Returns name
     * @return Name
     */
    std::string getName() const;

    /**
     * Returns wheather this ControlAspect is the passed one.
     * @param mode ControlAspect to compare with.
     * @return Are the two ControlAspects equal to each other?
     */
    bool equals(const ControlAspectPtr &mode) const;

    /**
     * Less operator.
     * Just for the sake of definiong an order on ControlAspect.
     */
    bool operator<(const ControlAspectPtr &compare);

    /**
     * Initializes control to desired mode.
     * @param name Name of the control mode
     * @param ID Identifier for this control mdoe
     * @return Control mode object, set to 0 (disabled)
     */
    static ControlAspectPtr create(const std::string &name,
            const std::string &ID);

    std::string print() const;

protected:
    /**
     * Name of the control mode.
     */
    std::string name;

    /**
     * Identifier of this control mode
     */
    std::string ID;

};

/**
 * Less operator.
 * Just for teh sake of definiong an order on ControlMode.
 */
//bool operator<(const ControlAspectPtr &source, const ControlAspectPtr &compare) {
//return (source->getID() < compare->getID());
//}

/**
 * Definition of default ControlAspect representing inactive control.
 */
static ControlAspectPtr noControl = ControlAspect::create(
        "Inactive / no Control", "NO_CONTROL");

/**
 * Control aspect representing Pose Control (cartesian task space control).
 */
static ControlAspectPtr poseControl = ControlAspect::create(
        "Cartesian Pose Control", "POSE_CONTROL");

/**
 * Definition of default ControlAspect representing Pose Control (cartesian
 * control).
 */
static const ControlAspectPtr forceControl = ControlAspect::create(
        "Cartesian Force Control", "FORCE_CONTROL");

/**
 * Definition of default ControlAspect representing Joint Position Control (joint
 * angles).
 */
static const ControlAspectPtr jointPositionControl = ControlAspect::create(
        "Joint Position Control", "POSITION_CONTROL");

/* Compliance */

/**
 * Definition of default ControlAspect representing Joint Compliance Control
 * (joint stiffness and damping for the joints).
 */
static const ControlAspectPtr jointComplianceControl = ControlAspect::create(
        "Joint Compliance Control", "ACTIVE_JOINT_COMPLIANCE");

/**
 * Definition of default ControlAspect representing Cartesian Compliance Control
 * (stiffness and damping in task space).
 */
static const ControlAspectPtr cartesianComplianceControl =
        ControlAspect::create("Cartesian Compliance Control",
                "ACTIVE_CARTESIAN_COMPLIANCE");

/**
 * Definition of default ControlAspect representing Joint Velocity Control.
 */
static const ControlAspectPtr jointVelocityControl = ControlAspect::create(
        "Joint Velocity Control", "VELOCITY_CONTROL");

/**
 * Definition of default ControlAspect representing Joint Torque Control.
 */
static const ControlAspectPtr jointTorqueControl = ControlAspect::create(
        "Joint Torque Control", "TORQUE_CONTROL");
}
