/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#include "ControlMode.h"

#include <sstream>
#include <algorithm>

using namespace std;

namespace rci {

ControlMode::ControlMode(string cname) :
    name(cname), contained() {
}

ControlMode::~ControlMode() {
}

std::string ControlMode::getName() const {
    return this->name;
}

bool ControlMode::equals(const ControlModePtr &mode) const {
    if (this->numAspects() != mode->numAspects()) {
        return false;
    }
    // Since aspects are sorted, we can just compare aspect by aspect
    for (unsigned int asp = 0; asp < this->numAspects(); ++asp) {
        if (this->contained[asp] != mode->contained[asp]) {
            return false;
        }
    }
    return true;
}

bool ControlMode::supports(const ControlAspectPtr &aspect) const {
    unsigned int num = this->numAspects();
    if ((num == 0) && (!aspect->equals(noControl))) {
        return false;
    }
    for (unsigned int asp = 0; asp < num; ++asp) {
        if (this->contained[asp]->equals(aspect)) {
            return true;
        }
    }
    return false;
}

void ControlMode::addControlAspect(ControlAspectPtr mode) {
    this->contained.push_back(mode);
    std::sort(this->contained.begin(), this->contained.end());
}

ControlModePtr ControlMode::combineWith(const ControlModePtr &mode,
        const std::string &nme) const {
    ControlMode *cm = new ControlMode(nme);

    // Add control aspects of passed control mode
    for (unsigned int asp = 0; asp < this->contained.size(); ++asp) {
        cm->addControlAspect(this->contained[asp]);
    }

    // Add my own control aspects
    for (unsigned int asp = 0; asp < mode->contained.size(); ++asp) {
        cm->addControlAspect(mode->contained[asp]);
    }

    return ControlModePtr(cm);
}

unsigned int ControlMode::numAspects() const {
    return this->contained.size();
}

std::string ControlMode::print() const {
    ostringstream outstream(ostringstream::out);
    outstream << "ControlMode<" << name << ">:" << endl;

    // Add control aspects of passed control mode
    for (unsigned int asp = 0; asp < this->contained.size(); ++asp) {
        outstream << "  " << this->contained[asp]->print() << endl;
    }

    return outstream.str();
}

ControlModePtr ControlMode::create(string name) {
    ControlModePtr mode(new ControlMode(name));
    return mode;
}

ControlModePtr ControlMode::create(string name, ControlAspectPtr aspect) {
    ControlModePtr mode(new ControlMode(name));
    mode->addControlAspect(aspect);
    return mode;
}

ControlModePtr ControlMode::create(string name, ControlAspectPtr aspect1,
        ControlAspectPtr aspect2) {
    ControlModePtr mode(new ControlMode(name));
    mode->addControlAspect(aspect1);
    mode->addControlAspect(aspect2);
    return mode;
}

}
