/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>

#include "rci/ControlMode.h"
#include "rci/dto/JointAngles.h"
#include "rci/dto/JointTorques.h"
#include "rci/dto/JointImpedance.h"
#include "rci/dto/JointVelocities.h"
#include "rci/dto/Pressure.h"
#include "rci/dto/Length.h"
#include "rci/dto/CartesianPose.h"

namespace rci {

class ControlledIF {

public:
	/**
	 * Controller converged
	 *
	 * Answers the question, if controller converged after last command.
	 */
	virtual bool isConverged() const = 0;

	/**
	 * Control mode getter
	 *
	 * @return Current control mode
	 */
	virtual ControlModePtr getControlMode() =0;

	/**
	 * Control mode setter
	 *
	 * @param controlMode Desired control mode
	 */
	virtual void setControlMode(ControlModePtr controlMode) =0;

protected:
	~ControlledIF() {
	}
};

/**
 * Interface indicating Joint Position Control
 *
 * Interface indicating Joint Position Control on a Node.
 */
class PositionControlledIF {

public:
	/**
	 * Sets position
	 *
	 * Sets a position on a joint/actuator (joint space control).
	 */
	virtual bool setJointPosition(JointAnglesPtr position) = 0;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual JointAnglesPtr getLastPositionCommand() const =0;

protected:
	~PositionControlledIF() {
	}
};

/**
 * Interface indicating Joint Velocity Control
 *
 * Interface indicating Joint Velocity Control on a Node.
 */
class VelocityControlledIF {

public:
	/**
	 * Sets position
	 *
	 * Sets a position on a joint/actuator (joint space control).
	 */
	virtual bool setJointVelocity(JointVelocitiesPtr velocity) = 0;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual JointVelocitiesPtr getLastVelocityCommand() const =0;

protected:
	~VelocityControlledIF() {
	}

};

/**
 * Interface indicating Joint Torque Control
 *
 * Interface indicating Joint Torque Control on a Node.
 */
class TorqueControlledIF {

public:
	/**
	 * Sets position
	 *
	 * Sets a position on a joint/actuator (joint space control).
	 */
	virtual bool setJointTorque(JointTorquesPtr position) = 0;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual JointTorquesPtr getLastTorqueCommand() const =0;

protected:
	~TorqueControlledIF() {
	}
};

/**
 * Interface indicating Joint Impedance Control
 *
 * Interface indicating Joint Impedance Control on a Node.
 */
class ImpedanceControlledIF {

public:
	/**
	 * Sets position
	 *
	 * Sets a position on a joint/actuator (joint space control).
	 */
	virtual bool setJointImpedance(JointImpedancePtr impedance) = 0;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual JointImpedancePtr getLastImpedanceCommand() const =0;

protected:
	~ImpedanceControlledIF() {
	}
};

/**
 * Interface indicating Pressure Control on an actuated node
 *
 * Interface indicating Pressure Control on an actuated node.
 */
class PressureControlledIF {

public:
	/**
	 * Sets position
	 *
	 * Sets a position on a joint/actuator (joint space control).
	 */
	virtual bool setPressure(PressurePtr pressure) = 0;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual PressurePtr getLastPressureCommand() const =0;

protected:
	~PressureControlledIF() {
	}
};

/**
 * Interface indicating Pressure Control on an actuated node
 *
 * Interface indicating Pressure Control on an actuated node.
 */
class LengthControlledIF {

public:
	/**
	 * Sets position
	 *
	 * Sets a position on a joint/actuator (joint space control).
	 */
	virtual bool setLength(LengthPtr Length) = 0;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual LengthPtr getLastLengthCommand() const =0;

protected:
	~LengthControlledIF() {
	}
};

/**
 * Interface indicating Cartesian Pose Control
 *
 * Interface indicating Cartesian Pose Control on a Node.
 */
class PoseControlledIF {

public:
	/**
	 * Sets a cartesian pose
	 *
	 * Sets a cartesian pose on a end-effector (task space control).
	 */
	virtual bool setPose(PosePtr pose) = 0;

	/**
	 * Returns latest pose command. Note, that this is the latest valid
	 * commanded pose. If the poe command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual PosePtr getLastPoseCommand() const =0;

protected:
	~PoseControlledIF() {
	}
};

}
