/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>

#include <boost/shared_ptr.hpp>

namespace rci {

/**
 * Helper methods for RCI.
 *
 * @ingroup DTO
 */
class RCIHelper {

public:

    /// Constructor
    RCIHelper(const std::string &name, const std::string &ID);

    /// Destructor
    virtual ~RCIHelper();

    /**
     * Merges several low-dimensional DTOs into one high-dimensional DTO.
     *
     * @param inputDTOs
     *          Vector of low-dimensional input DTOs to merge
     *
     * @return Merged high-dimensional DTO
     */
    template<class DTOType>
    static boost::shared_ptr<DTOType> mergeDTOs(
            const std::vector<boost::shared_ptr<DTOType> > inputDTOs) {
        // Calculate output dimension
        unsigned int outputdimension = 0;
        for (unsigned int inputdto = 0; inputdto < inputDTOs.size();
                ++inputdto) {
            for (unsigned int inputdim = 0;
                    inputdim < inputDTOs[inputdto]->getDimension();
                    ++inputdim) {
                ++outputdimension;
            }
        }

        // Create and return merged DTOs
        boost::shared_ptr<DTOType> merged = DTOType::create(outputdimension);
        RCIHelper::mergeDTOs(merged, inputDTOs);
        return merged;
    }

    /**
     * Merges several low-dimensional DTOs into one high-dimensional DTO.
     *
     * @param outputDTO
     *          High-dimensional output DTO
     * @param inputDTOs
     *          Vector of low-dimensional input DTOs to merge
     */
    template<class DTOType>
    static void mergeDTOs(boost::shared_ptr<DTOType> outputDTO,
            const std::vector<boost::shared_ptr<DTOType> > inputDTOs) {
        unsigned int outputcursor = 0;

        for (unsigned int inputdto = 0; inputdto < inputDTOs.size();
                ++inputdto) {
            for (unsigned int inputdim = 0;
                    inputdim < inputDTOs[inputdto]->getDimension();
                    ++inputdim) {
                outputDTO->setValue(outputcursor++,
                        inputDTOs[inputdto]->asDouble(inputdim));
            }
        }

    }

private:
    RCIHelper(const RCIHelper& RCIHelper);
};

}
