/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include "SensingIF.h"

namespace rci {

/**
 * Base sensing interface
 */
class Sensing: public SensingIF {

public:

	/**
	 * Update sensor data
	 *
	 * This method might not be required depending on the actual sensor.
	 */
	virtual void update();

	/**
	 * Read sensor data
	 *
	 * Returns a current readout of the sensor values. This is not wise to use,
	 * use methods with semantics instead.
	 */
	virtual cca::DataTransferObjectPtr get() const;

protected:
	Sensing();
	virtual ~Sensing();
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense joint angles (encoder value).
 */
class PositionSensing: public PositionSensingIF {

public:
	/**
	 * Returns joint position (encoder value)
	 */
	virtual JointAnglesPtr getJointPosition() const;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateJointPosition(JointAnglesPtr angle);

protected:
	PositionSensing(unsigned int dimension = 1);
	virtual ~PositionSensing();

	/**
	 * Latest sensor read-out
	 */
	JointAnglesPtr _latestJointPosition;
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense joint velocities (encoder value or derived).
 */
class VelocitySensing: public VelocitySensingIF {

public:
	/**
	 * Returns joint velocity
	 */
	virtual JointVelocitiesPtr getVelocity() const;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateJointVelocity(JointVelocitiesPtr vel);

protected:
	VelocitySensing(unsigned int dimension = 1);
	virtual ~VelocitySensing();

	/**
	 * Latest sensor read-out
	 */
	JointVelocitiesPtr _latestJointVelocity;
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense pressure (for example a vault).
 */
class PressureSensing: public PressureSensingIF {

public:
	/**
	 * Returns pressure values
	 */
	virtual PressurePtr getPressure() const;

protected:
	PressureSensing(unsigned int dimension = 1);
	virtual ~PressureSensing();

	/**
	 * Latest sensor read-out
	 */
	PressurePtr _latestPressureValues;
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a length value.
 */
class LengthSensing: public LengthSensingIF {

public:
	/**
	 * Returns length values
	 */
	virtual LengthPtr getLength() const;

protected:
	LengthSensing(unsigned int dimension = 1);
	virtual ~LengthSensing();

	/**
	 * Latest sensor read-out
	 */
	LengthPtr _latestLengthValues;
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense (cartesian) forces.
 */
class ForceSensing: public ForceSensingIF {

public:
	/**
	 * Returns absolute forces
	 *
	 * @return 3-dimensional force
	 */
	virtual ForcesPtr getForces() const;

	/**
	 * Updates internal representation of forces
	 */
	virtual void updateForces(ForcesPtr force);

protected:
	ForceSensing();
	virtual ~ForceSensing();

	/**
	 * Latest sensor read-out
	 */
	ForcesPtr forces;
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense joint torques.
 */
class TorqueSensing: public TorqueSensingIF {

public:
	/**
	 * Returns joint torque
	 *
	 * @return Joint torque
	 */
	virtual JointTorquesPtr getTorque() const;

	/**
	 * Updates internal representation of torques
	 */
	virtual void updateTorque(JointTorquesPtr torque);

protected:
	TorqueSensing(unsigned int dimension = 1);
	virtual ~TorqueSensing();

	/**
	 * Latest sensor read-out
	 */
	JointTorquesPtr torques;
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense (cartesian) forces.
 */
class PowerSensing: public PowerSensingIF {

public:
	/**
	 * Returns current power consumption
	 *
	 * @return Current power consumption
	 */
	virtual PowerPtr getPower() const;

protected:
	PowerSensing(unsigned int dimension = 1);
	virtual ~PowerSensing();

	/**
	 * Latest sensor read-out
	 */
	PowerPtr power;
};

}
