/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include "rci/SensingCartesianIF.h"

namespace rci {

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a full pose in cartesian space or it`s translational or rotational
 * part.
 */
class OrientationSensing: public OrientationSensingIF {

public:
	/**
	 * Returns absolute Orientation in cartesian task space.
	 *
	 * @return 3-dimensional task space Orientation
	 */
	virtual OrientationPtr getOrientation() const;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateOrientation(OrientationPtr orient);

protected:
	OrientationSensing();
	virtual ~OrientationSensing();

	/**
	 * Latest sensor read-out
	 */
	OrientationPtr orientation;

};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a full pose in cartesian space or it`s translational or rotational
 * part.
 */
class TranslationSensing: public TranslationSensingIF {

public:

	/**
	 * Returns absolute Translation in cartesian task space.
	 *
	 * @return 3-dimensional task space position
	 */
	virtual TranslationPtr getTranslation() const;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateTranslation(TranslationPtr translat);

protected:
	TranslationSensing();
	virtual ~TranslationSensing();

	/**
	 * Latest sensor read-out
	 */
	TranslationPtr translation;

};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a full pose in cartesian space or it`s translational or rotational
 * part.
 */
class PoseSensing: public PoseSensingIF,
		public TranslationSensing,
		public OrientationSensing {

public:
	/**
	 * Returns absolut Pose in cartesian task space.
	 *
	 * @return 6-dimensional task space Pose
	 */
	virtual PosePtr getPose() const;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updatePose(PosePtr pose);

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updatePose(TranslationPtr transl, OrientationPtr orient);

protected:
	PoseSensing();
	virtual ~PoseSensing();
};

/**
 * Velocity
 */

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional velocity of an object in cartesian space or it`s
 * translational or rotational part.
 */
class TranslationalVelocitySensing: public TranslationalVelocitySensingIF {

public:

	/**
	 * Returns translational velocity in cartesian task space.
	 *
	 * @return 3-dimensional task space position
	 */
	virtual TranslationalVelocityPtr getTranslationalVelocity() const;

	/**
	 * Updates sensor value for the translational part of the measured cartesian
	 * velocity.
	 *
	 * @param tvel
	 *          Translational velocity
	 */
	virtual void updateTranslationalVelocity(TranslationalVelocityPtr tvel);

protected:
	TranslationalVelocitySensing();
	virtual ~TranslationalVelocitySensing();

	/**
	 * Latest sensor read-out
	 */
	TranslationalVelocityPtr veltranslational;

};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional velocity of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianVelocity (joined transl. and rotat.)
 */
class RotationalVelocitySensing: public RotationalVelocitySensingIF {

public:

	/**
	 * Returns rotational velocity in cartesian task space.
	 *
	 * @return 3-dimensional task space Orientation
	 */
	virtual RotationalVelocityPtr getRotationalVelocity() const;

	/**
	 * Updates sensor value for the rotational part of the measured cartesian
	 * velocity.
	 *
	 * @param rvel
	 *          Rotational velocity
	 */
	virtual void updateRotationalVelocity(RotationalVelocityPtr rvel);

protected:
	RotationalVelocitySensing();
	virtual ~RotationalVelocitySensing();

	/**
	 * Latest sensor read-out
	 */
	RotationalVelocityPtr velrotational;

};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional velocity of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianVelocity (joined transl. and rotat.)
 */
class CartesianVelocitySensing: public CartesianVelocitySensingIF,
		public TranslationalVelocitySensing,
		public RotationalVelocitySensing {

public:

	/**
	 * Updates sensor value for the measured cartesian velocity.
	 *
	 * @param transvel
	 *          Translational velocity
	 * @param rotvel
	 *          Rotational velocity
	 */
	virtual void updateVelocity(TranslationalVelocityPtr transvel,
			RotationalVelocityPtr rotvel);

	virtual VelocityPtr getVelocity();

protected:
	CartesianVelocitySensing();
	virtual ~CartesianVelocitySensing();

};

/**
 * Acceleration
 */

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional acceleration of an object in cartesian space or it`s
 * translational or rotational part.
 */
class TranslationalAccelerationSensing: public TranslationalAccelerationSensingIF {

public:

	/**
	 * Returns translational acceleration in cartesian task space.
	 *
	 * @return 3-dimensional task space position
	 */
	virtual TranslationalAccelerationPtr getTranslationalAcceleration() const;

	/**
	 * Updates sensor value for the translational part of the measured cartesian
	 * acceleration.
	 *
	 * @param tacc
	 *          Translational acceleration
	 */
	virtual void updateTranslationalAcceleration(
			TranslationalAccelerationPtr tacc);

protected:
	TranslationalAccelerationSensing();
	virtual ~TranslationalAccelerationSensing();

	/**
	 * Latest sensor read-out
	 */
	TranslationalAccelerationPtr acctranslational;

};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional acceleration of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianAcceleration (joined transl. and rotat.)
 */
class RotationalAccelerationSensing: public RotationalAccelerationSensingIF {

public:

	/**
	 * Returns rotational acceleration in cartesian task space.
	 *
	 * @return 3-dimensional task space Orientation
	 */
	virtual RotationalAccelerationPtr getRotationalAcceleration() const;

	/**
	 * Updates sensor value for the rotational part of the measured cartesian
	 * acceleration.
	 *
	 * @param racc
	 *          Rotational acceleration
	 */
	virtual void updateRotationalAcceleration(RotationalAccelerationPtr racc);

protected:
	RotationalAccelerationSensing();
	virtual ~RotationalAccelerationSensing();

	/**
	 * Latest sensor read-out
	 */
	RotationalAccelerationPtr accrotational;

};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional acceleration of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianAcceleration (joined transl. and rotat.)
 */
class CartesianAccelerationSensing: public CartesianAccelerationSensingIF,
		public TranslationalAccelerationSensing,
		public RotationalAccelerationSensing {

public:

	/**
	 * Updates sensor value for the measured cartesian acceleration.
	 *
	 * @param transacc
	 *          Translational acceleration
	 * @param rotacc
	 *          Rotational acceleration
	 */
	virtual void updateAcceleration(TranslationalAccelerationPtr transacc,
			RotationalAccelerationPtr rotacc);

	virtual AccelerationPtr getAcceleration();

protected:
	CartesianAccelerationSensing();
	virtual ~CartesianAccelerationSensing();

};

}
