/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <cca/dto/DataTransferObject.h>

#include "rci/dto/CartesianPose.h"
#include "rci/dto/CartesianVelocity.h"
#include "rci/dto/CartesianAcceleration.h"

namespace rci {

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a full pose in cartesian space or it`s translational or rotational
 * part.
 */
class OrientationSensingIF {

public:
	/**
	 * Returns absolute Orientation in cartesian task space.
	 *
	 * @return 3-dimensional task space Orientation
	 */
	virtual OrientationPtr getOrientation() const = 0;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateOrientation(OrientationPtr orient) = 0;

protected:
	~OrientationSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a full pose in cartesian space or it`s translational or rotational
 * part.
 */
class TranslationSensingIF {

public:

	/**
	 * Returns absolute Translation in cartesian task space.
	 *
	 * @return 3-dimensional task space position
	 */
	virtual TranslationPtr getTranslation() const = 0;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateTranslation(TranslationPtr translat) = 0;

protected:
	~TranslationSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a full pose in cartesian space or it`s translational or rotational
 * part.
 */
class PoseSensingIF {

public:
	/**
	 * Returns absolut Pose in cartesian task space.
	 *
	 * @return 6-dimensional task space Pose
	 */
	virtual PosePtr getPose() const = 0;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updatePose(PosePtr pose) = 0;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updatePose(TranslationPtr transl, OrientationPtr orient) = 0;

protected:
	~PoseSensingIF() {
	}
};

/**
 * Velocity
 */

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional velocity of an object in cartesian space or it`s
 * translational or rotational part.
 */
class TranslationalVelocitySensingIF {

public:

	/**
	 * Returns translational velocity in cartesian task space.
	 *
	 * @return 3-dimensional task space position
	 */
	virtual TranslationalVelocityPtr getTranslationalVelocity() const = 0;

	/**
	 * Updates sensor value for the translational part of the measured cartesian
	 * velocity.
	 *
	 * @param tvel
	 *          Translational velocity
	 */
	virtual void updateTranslationalVelocity(TranslationalVelocityPtr tvel) = 0;

protected:
	~TranslationalVelocitySensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional velocity of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianVelocity (joined transl. and rotat.)
 */
class RotationalVelocitySensingIF {

public:

	/**
	 * Returns rotational velocity in cartesian task space.
	 *
	 * @return 3-dimensional task space Orientation
	 */
	virtual RotationalVelocityPtr getRotationalVelocity() const = 0;

	/**
	 * Updates sensor value for the rotational part of the measured cartesian
	 * velocity.
	 *
	 * @param rvel
	 *          Rotational velocity
	 */
	virtual void updateRotationalVelocity(RotationalVelocityPtr rvel) = 0;

protected:
	~RotationalVelocitySensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional velocity of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianVelocity (joined transl. and rotat.)
 */
class CartesianVelocitySensingIF {

public:

	/**
	 * Updates sensor value for the measured cartesian velocity.
	 *
	 * @param transvel
	 *          Translational velocity
	 * @param rotvel
	 *          Rotational velocity
	 */
	virtual void updateVelocity(TranslationalVelocityPtr transvel,
			RotationalVelocityPtr rotvel) = 0;

	virtual VelocityPtr getVelocity() = 0;

protected:
	~CartesianVelocitySensingIF() {
	}

};

/**
 * Acceleration
 */

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional acceleration of an object in cartesian space or it`s
 * translational or rotational part.
 */
class TranslationalAccelerationSensingIF {

public:

	/**
	 * Returns translational acceleration in cartesian task space.
	 *
	 * @return 3-dimensional task space position
	 */
	virtual TranslationalAccelerationPtr getTranslationalAcceleration() const = 0;

	/**
	 * Updates sensor value for the translational part of the measured cartesian
	 * acceleration.
	 *
	 * @param tacc
	 *          Translational acceleration
	 */
	virtual void updateTranslationalAcceleration(
			TranslationalAccelerationPtr tacc) = 0;

protected:
	~TranslationalAccelerationSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional acceleration of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianAcceleration (joined transl. and rotat.)
 */
class RotationalAccelerationSensingIF {

public:

	/**
	 * Returns rotational acceleration in cartesian task space.
	 *
	 * @return 3-dimensional task space Orientation
	 */
	virtual RotationalAccelerationPtr getRotationalAcceleration() const = 0;

	/**
	 * Updates sensor value for the rotational part of the measured cartesian
	 * acceleration.
	 *
	 * @param racc
	 *          Rotational acceleration
	 */
	virtual void updateRotationalAcceleration(
			RotationalAccelerationPtr racc) = 0;

protected:
	~RotationalAccelerationSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense the 6-dimensional acceleration of an object in cartesian space or it`s
 * translational or rotational part.
 *
 * @todo Also update and return CartesianAcceleration (joined transl. and rotat.)
 */
class CartesianAccelerationSensingIF {

public:

	/**
	 * Updates sensor value for the measured cartesian acceleration.
	 *
	 * @param transacc
	 *          Translational acceleration
	 * @param rotacc
	 *          Rotational acceleration
	 */
	virtual void updateAcceleration(TranslationalAccelerationPtr transacc,
			RotationalAccelerationPtr rotacc) = 0;

	virtual AccelerationPtr getAcceleration() = 0;

protected:
	~CartesianAccelerationSensingIF() {
	}

};

}
