/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <cca/dto/DataTransferObject.h>

#include "SensingCartesian.h"

#include "rci/dto/JointAngles.h"
#include "rci/dto/JointVelocities.h"
#include "rci/dto/CartesianForces.h"
#include "rci/dto/Pressure.h"
#include "rci/dto/Length.h"
#include "rci/dto/JointTorques.h"
#include "rci/dto/Power.h"

namespace rci {

/**
 * Base sensing interface
 */
class SensingIF {

public:

	/**
	 * Update sensor data
	 *
	 * This method might not be required depending on the actual sensor.
	 */
	virtual void update() =0;

	/**
	 * Read sensor data
	 *
	 * Returns a current readout of the sensor values. This is not wise to use,
	 * use methods with semantics instead.
	 */
	virtual cca::DataTransferObjectPtr get() const =0;

protected:
	~SensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense joint angles (encoder value).
 */
class PositionSensingIF {

public:
	/**
	 * Returns joint position (encoder value)
	 */
	virtual JointAnglesPtr getJointPosition() const = 0;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateJointPosition(JointAnglesPtr angle) =0;

protected:
	~PositionSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense joint velocities (encoder value or derived).
 */
class VelocitySensingIF {

public:
	/**
	 * Returns joint velocity
	 */
	virtual JointVelocitiesPtr getVelocity() const = 0;

	/**
	 * Updates internal representation of joint position / angle
	 */
	virtual void updateJointVelocity(JointVelocitiesPtr vel)=0;

protected:
	~VelocitySensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense pressure (for example a vault).
 */
class PressureSensingIF {

public:
	/**
	 * Returns pressure values
	 */
	virtual PressurePtr getPressure() const =0;

protected:
	~PressureSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense a length value.
 */
class LengthSensingIF {

public:
	/**
	 * Returns length values
	 */
	virtual LengthPtr getLength() const =0;

protected:
	~LengthSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense (cartesian) forces.
 */
class ForceSensingIF {

public:
	/**
	 * Returns absolute forces
	 *
	 * @return 3-dimensional force
	 */
	virtual ForcesPtr getForces() const =0;

	/**
	 * Updates internal representation of forces
	 */
	virtual void updateForces(ForcesPtr force)=0;

protected:
	~ForceSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense joint torques.
 */
class TorqueSensingIF {

public:
	/**
	 * Returns joint torque
	 *
	 * @return Joint torque
	 */
	virtual JointTorquesPtr getTorque() const =0;

	/**
	 * Updates internal representation of torques
	 */
	virtual void updateTorque(JointTorquesPtr torque) =0;

protected:
	~TorqueSensingIF() {
	}
};

/**
 * By implementing this interface, a node is characterized as being able to
 * sense (cartesian) forces.
 */
class PowerSensingIF {

public:
	/**
	 * Returns current power consumption
	 *
	 * @return Current power consumption
	 */
	virtual PowerPtr getPower() const = 0;

protected:
	~PowerSensingIF() {
	}
};

}
