/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <vector>
#include <iostream>
#include <boost/shared_ptr.hpp>

#include "rci/ResourceNode.h"

namespace rci {

class Synchronizer;
typedef boost::shared_ptr<Synchronizer> SynchronizerPtr;

/**
 * Synchronizer
 * Different synchronizers may be for example:
 * * Update with a given frequency
 * ** Send data every time
 * ** Send only if new data
 * * Update only if new data (event-based)
 * * Triggered from robot side (e.g. FRI)
 */
class Synchronizer {
public:
    Synchronizer(const std::string &name);
    virtual ~Synchronizer();

    /**
     * Synchronous processing (synchronisation of commands and sensor values
     * with the robot). After calling this, you can safely access new sensor
     * values.
     * Generic implementation of this method calls processAsync and then
     * waitForProcessAsync.
     */
    virtual void process();

    /**
     * Asynchronous processing (synchronisation of commands and sensor values
     * with the robot). This method will return immediately, you will have to
     * waitForProcessAsync to finish to have new sensor values and safely access
     * them.
     */
    virtual void processAsync() = 0;

    /**
     * Waits until asynchronous processing (synchronisation of commands and
     * sensor values with the robot) is done.
     * You will need for this to know that new sensor values arrived and you can
     * read them savely.
     */
    virtual void waitForProcessAsync() = 0;

    /**
     * Starts synchronizing. For some backends this has to be stopped, e.g. for
     * control mode switches.
     */
    virtual bool start();

    /**
     * Stops synchronizing, e.g. to have time for control mode switches.
     */
    virtual bool stop();

    /**
     * Returns, if the synchronizer is (too) busy right now. This
     * is important to know in case of robots where control has to be paused in
     * order to make a certain action (e.g. control mode switch).
     */
    virtual bool tooBusy() const;

    /**
     *
     */
    friend std::ostream
    & operator<<(std::ostream& os, const Synchronizer& val);

protected:

    /**
     * Only true for the first time
     */
    bool _onlyReceive;

    /**
     *
     */
    bool busy;

    /**
     *
     */
    std::string name;
};

}
