/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>
#include <sstream>

#include <rsc/math/MathUtils.h>

#include <nemo/Matrix.h>

#include "rci/dto/CartesianValues.h"

namespace rci {

class Pose;
typedef boost::shared_ptr<Pose> PosePtr;
class Orientation;
typedef boost::shared_ptr<Orientation> OrientationPtr;
class Translation;
typedef boost::shared_ptr<Translation> TranslationPtr;

class CartesianVelocity;
typedef boost::shared_ptr<CartesianVelocity> CartesianVelocityPtr;
class CartesianAcceleration;
typedef boost::shared_ptr<CartesianAcceleration> CartesianAccelerationPtr;

/**
 * Domain object, representing a cartesian translation in all three dimensions.
 * Default units are (\f$m, m, m\f$).
 * @ingroup DTO
 */
class Translation: public CartesianTranslational {

public:
	Translation();
	Translation(double valx, double valy, double valz);
	Translation(const nemo::RealVector& values);
	Translation(const Translation& translation);
	Translation(const TranslationPtr& translation);

	static TranslationPtr fromMeters(double x, double y, double z);

	/**
	 * Printing the translation object
	 *
	 * @return Textual representation of this translation.
	 */
	virtual std::string print() const;
};

/**
 * Domain object, representing an orientation or rotational transformation in
 * cartesian space. We represent orientation as quaternions, for reference see
 * http://en.wikipedia.org/wiki/Quaternions_and_spatial_rotation
 * @ingroup DTO
 */
class Orientation: public CartesianRotational {

public:
	Orientation();
	Orientation(double q0, double q1, double q2, double q3);
	Orientation(const nemo::RealVector& values);
	Orientation(const Orientation& orientation);
	Orientation(const OrientationPtr& orientation);
	Orientation(const ::boost::math::quaternion<double>& quaternion);
	Orientation(double rxx, double rxy, double rxz, double ryx, double ryy,
			double ryz, double rzx, double rzy, double rzz);
	Orientation(const nemo::RealMatrix& rotMat);

    /**
     * Create orientation from given euler angles. Convention is euler angles in
     * order x, y, z - right-handed.
     *
     * @param rotX
     *          Rotation angle around the x-axis
     * @param rotY
     *          Rotation angle around the y-axis
     * @param rotZ
     *          Rotation angle around the z-axis
     */
    static OrientationPtr
    fromEulerAngles(double rotX, double rotY, double rotZ);

	/**
	 * Create orientation from given values in axis-angle representation.
	 * Expects rotation angle and axis as unit vector {x, y, z}.
	 *
	 * @param angle
	 * 			Rotation angle
	 * @param vecX
	 * 			X-Component of rotation axis
	 * @param vecY
	 * 			Y-Component of rotation axis
	 * @param vecZ
	 * 			Z-Component of rotation axis
	 */
	static OrientationPtr
	fromAxisAngle(double angle, double vexX, double vecY, double vecZ);

	/**
	 * Create orientation from given values in compact axis-angle representation.
	 * Expects rotation as axis {x, y, z}, multiplied by rotation angle.
	 */
	static OrientationPtr
	fromCompactAxisAngle(double x, double y, double z);

	/**
	 * Create orientation from
	 */
	static OrientationPtr
	fromQuaternion(double q0, double q1, double q2, double q3);

	/**
	 * Create orientation from
	 */
	static OrientationPtr
	fromQuaternion(const ::boost::math::quaternion<double>& quaternion);

	static OrientationPtr
	fromRotationMatrix(double rxx, double rxy, double rxz, double ryx,
			double ryy, double ryz, double rzx, double rzy, double rzz);

	static OrientationPtr
	fromRotationMatrix(const nemo::RealMatrix& r);
	
	/**
	 * Apply a rotation to this orientation/rotation.
	 */
	virtual OrientationPtr rotateBy(const OrientationPtr& rot) const;

	/**
	 * Printing the orientation object
	 * @return Textual representation of this orientation.
	 */
	virtual std::string print() const;
};

/**
 * Domain object, representing a full cartesian pose, consisting of
 * translational transformation and orientation. Default units are (\f$m, m, m,
 * rad, rad, rad\f$).
 *
 * @ingroup DTO
 */
class Pose: public CartesianValue {

public:
	Pose();
    Pose(double valx, double valy, double valz, double vala, double valb,
            double valc, double vald);
    Pose(double valx, double valy, double valz, double a, double b,
            double c);
	Pose(const nemo::RealVector& values);
	Pose(const Pose& pose);
	Pose(const TranslationPtr& transl, const OrientationPtr& rot);

	/**
	 * Returns just the 3-dimensional translational part of this Pose.
	 *
	 * @return Translational part of this Pose.
	 */
	virtual TranslationPtr getTranslation() const;

	/**
	 * Returns just the rotational part of this Pose.
	 *
	 * @return Rotational part of this Pose.
	 */
	virtual OrientationPtr getOrientation() const;

	/**
	 * Domain specific getter for 1st translatory value.
	 *
	 * @return 1st translatory value in meter
	 */
	virtual double x() const;

	/**
	 * Domain specific getter for 2nd translatory value.
	 *
	 * @return 2nd translatory value in meter
	 */
	virtual double y() const;

	/**
	 * Domain specific getter for 3rd translatory value.
	 *
	 * @return 3rd translatory value in meter
	 */
	virtual double z() const;

	/**
	 * First quaternion component
	 * @return First quaternion component
	 */
	virtual double q0() const;

	/**
	 * Second quaternion component
	 *
	 * @return Second quaternion component
	 */
	virtual double q1() const;

	/**
	 * Third quaternion component
	 *
	 * @return Third quaternion component
	 */
	virtual double q2() const;

	/**
	 * Fourth quaternion component
	 *
	 * @return Fourth quaternion component
	 */
	virtual double q3() const;

	/**
	 * First euler angle from x, y, z - right-handed.
	 *
	 * @return Euler x-angle from x, y, z - right-handed. (\fi$rad\fi$)
	 */
	virtual double a() const;

	/**
	 * Second euler angle from x, y, z - right-handed.
	 *
	 * @return Euler y-angle from x, y, z - right-handed. (\fi$rad\fi$)
	 */
	virtual double b() const;

	/**
	 * Third euler angle from x, y, z - right-handed.
	 *
	 * @return Euler z-angle from x, y, z - right-handed. (\fi$rad\fi$)
	 */
	virtual double c() const;

	/**
	 * Shared Pointer
	 */
	static PosePtr fromMetersAndRadians(double x, double y, double z, double q0,
			double q1, double q2, double q3);
	static PosePtr fromMetersAndRadians(double x, double y, double z, double a,
			double b, double c);

	/**
	 * Creates pose from homogeneous translation. We expect only the three top
	 * rows of the 4x4 matrix, the last row (0, 0, 0, 1) is not necessary.
	 * Provide matrix like:
	 * |    rxx     rxy     rxz     tx  |
	 * |    ryx     ryy     ryz     ty  |
	 * |    rzx     rzy     rzz     tz  |
	 *
	 * @param rxx
	 *          Top-left matrix value
	 *
	 * @return Shared pointer to newly created pose
	 */
	static PosePtr fromMetersAndRadiansHomogeneous(double rxx, double rxy,
			double rxz, double tx, double ryx, double ryy, double ryz,
			double ty, double rzx, double rzy, double rzz, double tz);

	/**
	 * Creates pose from homogeneous translation matrix. We expect a 3x4 matrix,
	 * which is only the three top rows of the 4x4 matrix, the last row
	 * (0, 0, 0, 1) is not necessary.
	 *
	 * @param homogeneous
	 *          Homogeneous transformation as 3x4 matrix like:
	 *          |    rxx     rxy     rxz     tx  |
	 *          |    ryx     ryy     ryz     ty  |
	 *          |    rzx     rzy     rzz     tz  |
	 *
	 * @return Shared pointer to newly created pose
	 */
	static PosePtr fromMetersAndRadiansHomogeneous(
			const nemo::RealMatrix& homogeneous);

	/**
	 * Printing the pose object
	 *
	 * @return Textual representation of this pose.
	 */
	virtual std::string print() const;
};

}
