/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#include "CartesianVelocity.h"

#include <math.h>

#include <boost/math/quaternion.hpp>

using namespace std;
using namespace nemo;
using namespace rsc::math;

namespace rci {

TranslationalVelocity::TranslationalVelocity() :
        CartesianTranslational() {
}

TranslationalVelocity::~TranslationalVelocity() {
}

TranslationalVelocity::TranslationalVelocity(double valx, double valy,
        double valz) :
        CartesianTranslational(valx, valy, valz) {
}

TranslationalVelocity::TranslationalVelocity(RealVector values) :
        CartesianTranslational(values) {
}

TranslationalVelocityPtr TranslationalVelocity::fromM_s(double x, double y,
        double z) {
    return TranslationalVelocityPtr(new TranslationalVelocity(x, y, z));
}

double TranslationalVelocity::m_s(unsigned int index) {
    return this->asDouble(index);
}

nemo::RealVector TranslationalVelocity::m_sVector() {
    return this->asDoubleVector();
}

std::string TranslationalVelocity::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "TranslationalVelocity<x=" << this->asDouble(0) << ",y="
            << this->asDouble(1) << ",z=" << this->asDouble(2) << ">(m/s)";
    return outstream.str();
}

RotationalVelocity::RotationalVelocity() :
        CartesianRotational() {
}

RotationalVelocity::~RotationalVelocity(){
}

RotationalVelocity::RotationalVelocity(double vala, double valb, double valc,
        double vald) :
        CartesianRotational(vala, valb, valc, vald) {
}

RotationalVelocity::RotationalVelocity(RealVector values) :
        CartesianRotational(values) {
}

RotationalVelocity::RotationalVelocity(const RotationalVelocity& orient) :
        CartesianRotational() {
    this->_values = RealVector(orient.asDouble(0), orient.asDouble(1),
            orient.asDouble(2));
}

RotationalVelocity::RotationalVelocity(
        const ::boost::math::quaternion<double> quaternion) :
        CartesianRotational(quaternion) {
}

std::string RotationalVelocity::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "RotationalVelocity<q0=" << this->q0() << ",q1=" << this->q1()
            << ",q2=" << this->q2() << ",q3=" << this->q3() << ">" << std::endl;
    return outstream.str();
}

RotationalVelocityPtr RotationalVelocity::fromEulerAngles(double rotX,
        double rotY, double rotZ) {
    double q0 = cos(rotX / 2) * cos(rotY / 2) * cos(rotZ / 2)
            + sin(rotX / 2) * sin(rotY / 2) * sin(rotZ / 2);
    double q1 = sin(rotX / 2) * cos(rotY / 2) * cos(rotZ / 2)
            - cos(rotX / 2) * sin(rotY / 2) * sin(rotZ / 2);
    double q2 = cos(rotX / 2) * sin(rotY / 2) * cos(rotZ / 2)
            + sin(rotX / 2) * cos(rotY / 2) * sin(rotZ / 2);
    double q3 = cos(rotX / 2) * cos(rotY / 2) * sin(rotZ / 2)
            - sin(rotX / 2) * sin(rotY / 2) * cos(rotZ / 2);

    return RotationalVelocityPtr(new RotationalVelocity(q0, q1, q2, q3));
}

RotationalVelocityPtr RotationalVelocity::fromAxisAngle(double angle,
        double vecX, double vecY, double vecZ) {

    if (fabs(angle) < epsilon()) {
        // Angle is zero, direction vector has no effect
        return RotationalVelocityPtr(new RotationalVelocity(1.0, 0.0, 0.0, 0.0));
    }

    double qx = vecX * sin(angle / 2);
    double qy = vecY * sin(angle / 2);
    double qz = vecZ * sin(angle / 2);
    double qw = cos(angle / 2);

    return RotationalVelocityPtr(new RotationalVelocity(qw, qx, qy, qz));
}

RotationalVelocityPtr RotationalVelocity::fromCompactAxisAngle(double aaX,
        double aaY, double aaZ) {
    double angle = sqrt(aaX * aaX + aaY * aaY + aaZ * aaZ);
    return RotationalVelocity::fromAxisAngle(angle, aaX / angle, aaY / angle,
            aaZ / angle);
}

RotationalVelocityPtr RotationalVelocity::fromQuaternion(double q0, double q1,
        double q2, double q3) {
    ::boost::math::quaternion<double> quaternion(q0, q1, q2, q3);
    return RotationalVelocityPtr(new RotationalVelocity(quaternion));
}

RotationalVelocityPtr RotationalVelocity::fromQuaternion(
        const ::boost::math::quaternion<double> quaternion) {
    return RotationalVelocityPtr(new RotationalVelocity(quaternion));
}

}
