/* ============================================================
 *
 * This file is a part of RCI project
 *
 * Copyright (C) 2011 by Arne Nordmann <anordman at cor-lab dot uni-bielefeld dot de>
 *
 * This file may be licensed under the terms of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by:
 *   CoR-Lab, Research Institute for Cognition and Robotics
 *     Bielefeld University
 *
 * ============================================================ */

#pragma once

#include <iostream>
#include <sstream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include "rci/dto/JointValues.h"

namespace rci {

class JointVelocities;
typedef boost::shared_ptr<JointVelocities> JointVelocitiesPtr;

/**
 * Domain object, representing Joint Velocities.
 * Default representation is in radian per second (\f$\frac{rad}{s}\f$),
 * according to the International System of Units (SI).
 * @ingroup DTO
 */
class JointVelocities: public JointValues {

public:
    JointVelocities();
    JointVelocities(nemo::RealVector values);
    JointVelocities(double value);

    /**
     * Creates one-dimensional JointVelocitiesPtr from a single double value.
     * You should not use this method, unless you exactly know what you are
     * doing. Try to rather use a more specific method.
     *
     * @param value
     *          Initial value.
     *
     * @return One-dimensional JointVelocitiesPtr
     */
    static JointVelocitiesPtr fromValue(double value);

    /**
     * Creates a JointVelocitiesPtr from a vector of double values.
     * You should not use this method, unless you exactly know what you are
     * doing. Try to rather use a more specific method.
     *
     * @param values
     *          Initial velocity values.
     *
     * @return JointVelocitiesPtr of the size of the given double vector.
     */
    static JointVelocitiesPtr fromValues(nemo::RealVector values);

    /**
     * Creates a one-dimensional JointVelocitiesPtr from a single \f$\frac{rad}{s}\f value.
     *
     * @param values
     *          Initial velocity value in \f$\frac{rad}{s}\f$
     *
     * @return One-dimensional JointVelocitiesPtr
     */
    static JointVelocitiesPtr fromRad_s(double value);

    /**
     * Creates a JointVelocitiesPtr from a vector of \f$\frac{rad}{s}\f values.
     *
     * @param values
     *          Initial velocity values in \f$\frac{rad}{s}\f$
     *
     * @return JointVelocitiesPtr of the size of the given vector.
     */
    static JointVelocitiesPtr fromRad_s(nemo::RealVector values);

    /**
     * Get all velocity values as double vector (in \f$\frac{rad}{s}\f$).
     *
     * @return Double vector (in \f$\frac{rad}{s}\f$)
     */
    virtual nemo::RealVector rad_sVector();

    /**
     * Get single velocity value in \f$\frac{rad}{s}\f.
     *
     * @param index
     *          Index of velocity value
     *
     * @return Single velocity value in \f$\frac{rad}{s}\f$
     */
    virtual double rad_s(unsigned int index);

    /**
     * Creates a one-dimensional JointVelocitiesPtr from a single
     * \f$\frac{deg}{s}\f$ value.
     *
     * @param values
     *          Initial velocity value in \f$\frac{deg}{s}\f$.
     *
     * @return One-dimensional JointVelocitiesPtr
     */
    static JointVelocitiesPtr fromDeg_s(double value);

    /**
     * Creates a JointVelocitiesPtr from a vector of \f$\frac{rad}{s}\f$ values.
     *
     * @param values
     *          Initial velocity values in \f$\frac{rad}{s}\f$.
     *
     * @return JointVelocitiesPtr of the size of the given vector.
     */
    static JointVelocitiesPtr fromDeg_s(nemo::RealVector values);

    /**
     * Get all velocity values as double vector (in \f$\frac{rad}{s}\f$).
     *
     * @return Double vector (in \f$\frac{rad}{s}\f$)
     */
    virtual nemo::RealVector deg_sVector();

    /**
     * Get single velocity value in \f$\frac{rad}{s}\f$.
     *
     * @param index
     *          Index of velocity value
     *
     * @return Single velocity value in \f$\frac{rad}{s}\f$
     */

    virtual double deg_s(unsigned int index);
    /**
     * Print function
     */
    virtual std::string print() const;
};

}
