#include <gtest/gtest.h>

#include <iostream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include "rci/dto/JointImpedance.h"

using namespace nemo;
using namespace rci;

class JointImpedanceTest: public ::testing::Test {
protected:

    JointImpedanceTest() :
            values(), ji(), onedim1(), onedim2(), onedim3() {

        values = RealVector(dim(6));
        values[0] = 1.12;
        values[1] = 1.2 * M_PI;
        values[2] = -1 * M_PI;
        values[3] = 2.12;
        values[4] = 2.2 * M_PI;
        values[5] = -2 * M_PI;

        onedim1 = JointImpedancePtr(new JointImpedance(1.11, 2.22));
        onedim2 = JointImpedancePtr(new JointImpedance(3.33, 4.44));
        onedim3 = JointImpedancePtr(new JointImpedance(5.55, 6.66));

        ji = JointImpedance(values);
    }

    virtual ~JointImpedanceTest() {
    }

    RealVector values;
    JointImpedance ji;

    JointImpedancePtr onedim1, onedim2, onedim3;
};

TEST_F(JointImpedanceTest, isEmptyInitially)
{
    EXPECT_EQ(0, JointImpedance().getDimension())
    ;}

TEST_F(JointImpedanceTest, testConstruction)
{
    // Check dimension
    JointImpedance j = JointImpedance(values);
    EXPECT_EQ(3, j.getDimension());
}

TEST_F(JointImpedanceTest, testCopyConstruction)
{
    // Check dimension
    const JointImpedance j(values);
    JointImpedance copy = j;

    EXPECT_EQ(j.getDimension(), copy.getDimension());

    EXPECT_DOUBLE_EQ(j.asDouble(0), copy.asDouble(0));
    EXPECT_DOUBLE_EQ(j.asDouble(2), copy.asDouble(2));
    EXPECT_DOUBLE_EQ(j.asDouble(3), copy.asDouble(3));
    EXPECT_DOUBLE_EQ(j.asDouble(5), copy.asDouble(5));
}

TEST_F(JointImpedanceTest, testCopyCcreation)
{
    // Check dimension
    const JointImpedance j(values);
    JointImpedancePtr copy = JointImpedance::copy(j);

    EXPECT_EQ(j.getDimension(), copy->getDimension());

    EXPECT_DOUBLE_EQ(j.asDouble(0), copy->asDouble(0));
    EXPECT_DOUBLE_EQ(j.asDouble(2), copy->asDouble(2));
    EXPECT_DOUBLE_EQ(j.asDouble(3), copy->asDouble(3));
    EXPECT_DOUBLE_EQ(j.asDouble(5), copy->asDouble(5));
}

TEST_F(JointImpedanceTest, testGetters)
{
    RealVector testvalues;

    // Check stiffness and damping getter
    EXPECT_DOUBLE_EQ(1.12, ji.getStiffness()[0].asDouble(0))
    ; EXPECT_DOUBLE_EQ(- M_PI, ji.getStiffness()[1].asDouble(0));
    EXPECT_DOUBLE_EQ(1.2 * M_PI, ji.getDamping()[0].asDouble(0));
    EXPECT_DOUBLE_EQ(2.12, ji.getDamping()[1].asDouble(0));

    // Check operator[]
    EXPECT_DOUBLE_EQ(1.12, ji[0].getStiffness()[0].asDouble(0));
    EXPECT_DOUBLE_EQ(2.12, ji[1].getDamping()[0].asDouble(0));
    EXPECT_DOUBLE_EQ(2.2 * M_PI, ji[2].getStiffness()[0].asDouble(0));
}

TEST_F(JointImpedanceTest, testMergingOneDimensionalDTOs)
{
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(values)); // 3-dim

    std::vector<JointImpedancePtr> inputDTOs = std::vector<JointImpedancePtr>();
    inputDTOs.push_back(onedim1);
    inputDTOs.push_back(onedim2);
    inputDTOs.push_back(onedim3);

    a->merge(inputDTOs);

    EXPECT_EQ(3, a->getDimension())
    ; EXPECT_DOUBLE_EQ(1.11, a->asDouble(0));
    EXPECT_DOUBLE_EQ(2.22, a->asDouble(1));
    EXPECT_DOUBLE_EQ(3.33, a->asDouble(2));
}

TEST_F(JointImpedanceTest, testMergingOneDimensionalDTOsWithTooLittleInputs)
{
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(values)); // 3-dim

    std::vector<JointImpedancePtr> inputDTOs = std::vector<JointImpedancePtr>();
    inputDTOs.push_back(onedim1);
    inputDTOs.push_back(onedim2);

    EXPECT_THROW(a->merge(inputDTOs), std::runtime_error)
    ;}

TEST_F(JointImpedanceTest, testMergingOneDimensionalDTOsWithTooManyInputs)
{
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(values)); // 3-dim

    std::vector<JointImpedancePtr> inputDTOs = std::vector<JointImpedancePtr>();
    inputDTOs.push_back(onedim1);
    inputDTOs.push_back(onedim2);
    inputDTOs.push_back(onedim3);
    inputDTOs.push_back(onedim3);

    EXPECT_THROW(a->merge(inputDTOs), std::runtime_error)
    ;}

TEST_F(JointImpedanceTest, testMergingMultiDimensionalDTOs)
{
    RealVector v8dim(dim(8), 0.0);
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(v8dim));
    JointImpedancePtr b = JointImpedancePtr(new JointImpedance(values));

    std::vector<JointImpedancePtr> inputDTOs = std::vector<JointImpedancePtr>();
    inputDTOs.push_back(b);
    inputDTOs.push_back(onedim1);

    a->merge(inputDTOs);

    EXPECT_EQ(4, a->getDimension())
    ; EXPECT_DOUBLE_EQ(1.12, a->asDouble(0));
    EXPECT_DOUBLE_EQ(1.2 * M_PI, a->asDouble(1));
    EXPECT_DOUBLE_EQ(-M_PI, a->asDouble(2));
    EXPECT_DOUBLE_EQ(2.2 * M_PI, a->asDouble(4));
    EXPECT_DOUBLE_EQ(-2 * M_PI, a->asDouble(5));
    EXPECT_DOUBLE_EQ(2.22, a->asDouble(7));
}

TEST_F(JointImpedanceTest, testMergingMultiDimensionalDTOsWithTooLittleInputs)
{
    RealVector v8dim(dim(8), 0.0);
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(v8dim));
    JointImpedancePtr b = JointImpedancePtr(new JointImpedance(values));

    std::vector<JointImpedancePtr> inputDTOs = std::vector<JointImpedancePtr>();
    inputDTOs.push_back(b);

    EXPECT_THROW(a->merge(inputDTOs), std::runtime_error)
    ;}

TEST_F(JointImpedanceTest, testMergingMultiDimensionalDTOsWithTooManyInputs)
{
    RealVector v8dim(dim(8), 0.0);
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(v8dim));
    JointImpedancePtr b = JointImpedancePtr(new JointImpedance(values));

    std::vector<JointImpedancePtr> inputDTOs = std::vector<JointImpedancePtr>();
    inputDTOs.push_back(b);
    inputDTOs.push_back(b);

    EXPECT_THROW(a->merge(inputDTOs), std::runtime_error)
    ;}

TEST_F(JointImpedanceTest, testSplitting)
{
    JointImpedancePtr a = JointImpedancePtr(new JointImpedance(values));
    std::vector<JointImpedancePtr> outputDTOs = a->split<JointImpedance>();

    EXPECT_EQ(3, outputDTOs.size())
    ; EXPECT_EQ(1, outputDTOs[0]->getDimension());

    EXPECT_DOUBLE_EQ(1.12, outputDTOs[0]->asDouble(0));
    EXPECT_DOUBLE_EQ(1.2 * M_PI, outputDTOs[0]->asDouble(1));

    EXPECT_DOUBLE_EQ(-1 * M_PI, outputDTOs[1]->asDouble(0));
    EXPECT_DOUBLE_EQ(2.12, outputDTOs[1]->asDouble(1));

    EXPECT_DOUBLE_EQ(2.2 * M_PI, outputDTOs[2]->asDouble(0));
    EXPECT_DOUBLE_EQ(-2 * M_PI, outputDTOs[2]->asDouble(01));
}
